package com.lrm.android.kurs4;

import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;

import org.opencv.android.BaseLoaderCallback;
import org.opencv.android.CameraBridgeViewBase;
import org.opencv.android.CameraBridgeViewBase.CvCameraViewListener;
import org.opencv.android.JavaCameraView;
import org.opencv.android.LoaderCallbackInterface;
import org.opencv.android.OpenCVLoader;
import org.opencv.core.Core;
import org.opencv.core.CvType;
import org.opencv.core.Mat;
import org.opencv.core.Point;
import org.opencv.core.Scalar;

public class MainActivity extends AppCompatActivity implements CvCameraViewListener {

    //Tag naszej klasy - dla wygody użycia.
    private static final String TAG = "Kurs4";

    /**
     * Obiekt odpowiedzialny za obsługę kamery i wyświetlanie jej podglądu.
     */
    private CameraBridgeViewBase mOpenCvCameraView = null;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        Log.d(TAG, "Creating and setting view");
        //Otwarcie odpowiedniej kamery (jakakolwiek, przednia, tylna).
        mOpenCvCameraView = new JavaCameraView(this, CameraBridgeViewBase.CAMERA_ID_FRONT);
        //Ustawienie podglądu kamery jako głównej zawartości okna aplikacji.
        setContentView(mOpenCvCameraView);
        //Zarejestrowanie funkcji odpowiedzialnych za obsługę zdarzeń związanych z kamerą.
        mOpenCvCameraView.setCvCameraViewListener(this);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    /**
     * Obiekt odpowiedzialny za obsługę zdarzeń dotyczących ładowania
     * biblioteki OpenCV. W tym wypadku modyfikujemy funkcję onManagerConnected,
     * aby wykonywała dodatkowe czynności.
     */
    private BaseLoaderCallback mLoaderCallback = new BaseLoaderCallback(this) {
        @Override
        public void onManagerConnected(int status) {
            //Jeśli wszystko poszło dobrze.
            if(status == LoaderCallbackInterface.SUCCESS) {
                Log.i(TAG, "OpenCV loaded successfully");
                //Włącz podgląd kamery.
                mOpenCvCameraView.enableView();
            }
            //Jeśli coś poszło źle.
            else {
                //Wywołaj oryginalną funkcję onManagerConnected.
                super.onManagerConnected(status);
            }
        }
    };

    /**
     * Funkcja wywoływana zawsze podczas wznawiania działania aplikacji,
     * a także podczas jej uruchamiania.
     */
    @Override
    public void onResume()
    {
        super.onResume();
        //Zlecenie załadowania biblioteki OpenCV. Po załadowaniu wywoływana jest
        //funkcja onManagerConnected z obiektu mLoaderCallback.
        OpenCVLoader.initAsync(OpenCVLoader.OPENCV_VERSION_2_4_11, this, mLoaderCallback);
    }

    /**
     * Funkcja wywoływana zawsze podczas zatrzymywania aplikacji,
     */
    @Override
    public void onPause()
    {
        super.onPause();
        //Jeśli kamera została otwarta to ją zwalniamy poprzez wyłączenie podglądu.
        if (mOpenCvCameraView != null) {
            mOpenCvCameraView.disableView();
        }
    }

    /**
     * Funkcja wywoływana po uruchomieniu podglądu kamery.
     * @param width Szerokość (w pikselach) klatek, które będą dostarczane.
     * @param height Wysokość (w pikselach) klatek, które będą dostarczane.
     */
    public void onCameraViewStarted(int width, int height) {

    }

    /**
     * Funkcja wywoływana po zatrzymaniu podglądu kamery.
     */
    public void onCameraViewStopped() {

    }

    /**
     * Funkcja wywoływana po otrzymaniu klatki z kamery.
     * @param inputFrame Klatka wejściowa w postaci macierzy.
     * @return Klatka, która zostanie wyświetlona na ekranie.
     */
    @Override
    public Mat onCameraFrame(Mat inputFrame) {
        int h = inputFrame.rows();
        int w = inputFrame.cols();
        int ch = inputFrame.channels();

        //TODO Tutaj uzupełnijcie kod modyfikujący obraz.



        //Koniec

        return inputFrame;
    }
}
